package com.usoft.fin.grpc.dal.mapper;

import com.usoft.fin.grpc.dal.entity.Repay;
import com.usoft.fin.grpc.dal.entity.vo.RepayVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface RepayMapper {
    /**
     * deleteByPrimaryKey
     * 
     * @param id
     * @return int
     */
    int deleteByPrimaryKey(Integer id);

    /**
     * insert
     * 
     * @param record
     * @return int
     */
    int insert(Repay record);

    /**
     * insertSelective
     * 
     * @param record
     * @return int
     */
    int insertSelective(Repay record);

    /**
     * selectByPrimaryKey
     * 
     * @param id
     * @return Repay
     */
    Repay selectByPrimaryKey(Integer id);

    /**
     * updateByPrimaryKeySelective
     * 
     * @param record
     * @return int
     */
    int updateByPrimaryKeySelective(Repay record);

    /**
     * updateByPrimaryKey
     * 
     * @param record
     * @return int
     */
    int updateByPrimaryKey(Repay record);

    /**
     * findByDrawCode
     *
     * @param drawApplyCode
     * @return
     */
    Repay findByDrawCode(String drawApplyCode);

    /**
     * 根据enuu统计数量
     * @param enuu
     * @return
     */
    int countByEnuu(int enuu);

    /**
     * 分页查询
     * @param start 当前页数
     * @param pageSize 每页数量
     * @param enuu
     * @return
     */
    List<RepayVO> selectListByEnuu(@Param("start") int start, @Param("pageSize") int pageSize, @Param("enuu") int enuu);

    /**
     * findByCode
     * @param code
     * @return
     */
    Repay findByCode(@Param("code") String code);
}